unit VIEWMETA;

interface

uses WinTypes, WinProcs, WObjects, Strings;

const
   mfs_FillWindow = 1;                   {Metafile Viewer Styles}
   mfs_Resource   = 2;

   mfn_ALDUS      = 1;                   {Metafile Viewer Notification Codes}
   mfn_MSOFT      = 2;
   mfn_Clicked    = 3;
   mfn_DblClicked = 4;

   mfm_PreserveAspect = wm_USER+1;       {Metafile Viewer internal messages}
   mfm_UseResource = wm_USER+2;


type
   PMetafileViewer = ^TMetafileViewer;
   TMetafileViewer = object(TControl)
      constructor Init(AParent: PWindowsObject;         {Parent Window}
                       AnID: integer;                   {Control ID}
                       AName: PChar;                    {Name of Resource or Disk File}
                       X, Y, W, H: integer;             {Position and Size}
                       InitPreserveAspect,              {Initial setting of Aspect Ratio}
                       MetafileIsAResource: boolean);   {Specifies if the Metafile control manipulates}
                                                        {   a resource or a disk file}

      constructor InitResource(AParent: PWindowsObject; ResourceID: integer);
      constructor Load(var S: TStream);
      destructor Done; virtual;
      function GetClassName: PChar; virtual;
      procedure Store(var S: TStream); virtual;
      procedure ToggleAspect; virtual;
      function AspectRatioPreserved: boolean; virtual;
      function IsAResource: boolean; virtual;
      procedure SetName(AName: PChar); virtual;
   end;


const
   RMetafileViewer: TStreamRec = (
      ObjType: 50000;
      VmtLink: Ofs(TypeOf(TMetafileViewer)^);
      Load: @TMetafileViewer.Load;
      Store: @TMetafileViewer.Store
   );

{**************************************************************}

implementation

{$F+}

   constructor TMetafileViewer.Init(AParent: PWindowsObject;
                               AnID: integer; AName: PChar;
                               X, Y, W, H: integer;
                               InitPreserveAspect,
                               MetafileIsAResource: boolean);
      begin
         TControl.Init(AParent, AnID, AName, X, Y, W, H);

         Attr.Style:= ws_Child + ws_Visible;
         if not(InitPreserveAspect) then
            Attr.Style:= Attr.Style + mfs_FillWindow;
         if MetaFileIsAResource then
            Attr.Style:= Attr.Style + mfs_Resource;
      end;

   constructor TMetafileViewer.InitResource(AParent: PWindowsObject;
                                            ResourceID: integer);
      begin
         TControl.InitResource(AParent, ResourceID);
      end;

   constructor TMetafileViewer.Load(var S: TStream);
      {The extra boolean values stored into the stream are required because }
      {for some reason, the Stream mechanism does not save the user defined }
      {portion of the window style properly.}
      var
         AResource,
         PreserveAspect : boolean;
         UserStyle      : longint;
      begin
         TControl.Load(S);
         S.Read(AResource, SizeOf(AResource));
         S.Read(PreserveAspect, SizeOf(PreserveAspect));
         UserStyle:= 0;
         if AResource then
            UserStyle:= UserStyle + mfs_Resource;
         if not(PreserveAspect) then
            UserStyle:= UserStyle + mfs_FillWindow;
         Attr.Style:= Attr.Style and not(mfs_Resource + mfs_FillWindow);
         Attr.Style:= Attr.Style + UserStyle;
      end;

   destructor TMetafileViewer.Done;
      begin
         TControl.Done;
      end;

   procedure TMetafileViewer.Store(var S: TStream);
      {see explanation given in the Load Method}
      var
         AResource,
         PreserveAspect : boolean;
         UserStyle      : longint;
      begin
         TControl.Store(S);
         UserStyle:= GetWindowLong(HWindow, gwl_Style);
         AResource:= ((UserStyle and mfs_Resource) = mfs_Resource);
         PreserveAspect:= ((UserStyle and mfs_FillWindow) <> mfs_FillWindow);
         S.Write(AResource, SizeOf(AResource));
         S.Write(PreserveAspect, SizeOf(PreserveAspect));
      end;

   function TMetafileViewer.GetClassName: PChar;
      begin
         GetClassName:= 'MetaFile Viewer';
      end;

   procedure TMetafileViewer.ToggleAspect;
      var
         PreserveAspect : boolean;
         UserStyle      : longint;
      begin
         UserStyle:= GetWindowLong(HWindow, gwl_Style);
         PreserveAspect:= ((UserStyle and mfs_FillWindow) <> mfs_FillWindow);
         SendMessage(HWindow, mfm_PreserveAspect,
            Word(Not(PreserveAspect)), 0);
         InvalidateRect(HWindow, nil, True);
      end;

   function TMetafileViewer.AspectRatioPreserved: boolean;
      var
         UserStyle : longint;
      begin
         UserStyle:= GetWindowLong(HWindow, gwl_Style);
         AspectRatioPreserved:= ((UserStyle and mfs_FillWindow) <> mfs_FillWindow);
      end;

   function TMetafileViewer.IsAResource: boolean;
      var
         UserStyle : longint;
      begin
         UserStyle:= GetWindowLong(HWindow, gwl_Style);
         IsAResource:= ((UserStyle and mfs_Resource) = mfs_Resource);
      end;

   procedure TMetafileViewer.SetName(AName: PChar);
      begin
         SendMessage(HWindow, wm_SetText, 0, longint(AName));
         InvalidateRect(HWindow, nil, True);
      end;

   procedure SetupMetaFileViewer; external 'METAVIEW';


begin
   SetupMetaFileViewer;
   RegisterType(RMetafileViewer);
end.

